;**************************************************************************************
;*
;* ICSXprog.asm
;* Ver 2.0
;* 2/4/2001
;*
;* Revision history:
;*
;* 14-Jan-1994: Ver 1.x: Original Fluffy DOS software (c) Ben Stragnell
;* 02-Apr-2001: Ver 2.0: Modified for use with IC-Prog1.4  Steven Willis
;* 20-Apr-2001: Ver 2.1: Added support for VDD on/off
;* 19-May-2001: Ver 2.2: Dual 38400/115200 BPS com port speed and LED indicator
;*
;* Acknowledgements:
;*
;* This modified version (Ver 2.x) is based on Ben Stragnell's 0928fluf.zip
;* SX Fluffy programmer. It is for serial port interface connection only
;* and should be used with IC-Prog by Bonny Gijzen.
;*
;* Legal stuff:
;*
;* This is copyright software, but I do not seek to profit from it.  I wrote
;* it in the hope that it will prove useful.  Feel free to modify the program,
;* but please DO NOT distribute modified versions.  Instead, send me your
;* modifications and improvements so that they may, at my discretion, be
;* included in a future release.
;*
;* The author of this program accepts no responsibility for damages that are
;* caused by this program and make NO WARRANTY or representation, either
;* express or implied, with respect to this software.  This software is
;* provided "AS IS" and you, its user, assume the entire risk when you use it.
;*
;* Although I cannot undertake to support this software, I would
;* welcome bug reports, suggestions or comments which should be
;* sent to:
;*
;*    Steven Willis,
;*    StevenWillis@semis.demo.co.uk
;*
;* Copyright (C) 1994 Ben Stragnell, Steven Willis.
;* This program is free software.  Permission is granted to use,
;* copy, or redistribute this program so long as it is not sold
;* for profit.
;*
;*
;**************************************************************************************
;*
;* PIC16F84 code for programming a Scenix SX chip or another PIC.
;*
;* Freq: 10MHz
;*
;*   B0 <-> SX OSC2 (or Target PIC RB6 [clock])
;*   B1  -> SX VDD  Voltage control (5V enable)
;*   B4 <-  Serial TX from PC
;*   B7 <-  38400/115200 BPS com port speed (5V or O/C=115200)
;*   A0  -> SX OSC1 Voltage Control (5V enable)
;*   A1  -> SX OSC1 Voltage Control (12V enable)
;*   A2  -> Serial RX to PC
;*
;**************************************************************************************

#include <p16f84.inc>

		LIST		P=16F84
		__config	h'3FF2'

DATALO		equ		h'10'		; File registers
DATAHI		equ		h'11'
BITCT		equ		h'12'
VALID		equ		h'13'
REPTCT		equ		h'14'
ICTR		equ		h'15'
TDATALO		equ		h'16'
TDATAHI		equ		h'17'
FROMSXLO	equ		h'18'
FROMSXHI	equ		h'19'
ERRORS		equ		h'1a'
PCCMD		equ		h'1b'
TMP		equ		h'1c'
SER_OUT		equ		h'1d'
SER_IN		equ		h'1e'
TMP2		equ		h'1f'
TMP3		equ		h'20'
STATREG		equ		h'21'		; Bit 0-POR complete, 1-115KBPS

PCCMD_NOP	equ		h'00'		; PC commands
PCCMD_OSC1	equ		h'01'
PCCMD_OSC2	equ		h'02'
PCCMD_ECHO	equ		h'03'
PCCMD_SX	equ		h'04'
PCCMD_PICINIT	equ		h'05'
PCCMD_PICCMD	equ		h'06'
PCCMD_PICCMDD	equ		h'07'
PCCMD_PICCLOSE	equ		h'08'
PCCMD_RESET	equ		h'09'
PCCMD_PICPROG	equ		h'0a'
PCCMD_VDD	equ		h'0b'

BIT_OSC1_5V	equ		0		; PortA bit assignments
BIT_OSC1_12V	equ		1
BIT_PCDO0	equ		2

BIT_OSC2	equ		0		; PortB bit assignments
BIT_VDD		equ		1
BIT_PCDI0	equ		4

ERROR_OK	equ		d'0'		; Error codes
ERROR_CMDINVLD	equ		d'1'
ERROR_BITCT	equ		d'2'
ERROR_TIMEOUT	equ		d'3'

;**************************************************************************************
;
		org		0			; Start of code
start		clrf		INTCON			; Reset Vector
		clrf		STATREG			; POR not complete yet
		goto		main

;**************************************************************************************
;
;interrupt handler... we're catching a falling edge on B0
;
; we have roughly 25 clocks before the data should be placed
; (1.5 128Kcycles=29 clocks - 4 for int latency).
;
		org		4			; Start of interrupt
intvec		movlw		d'7'			; 4
		movwf		ICTR			; 5
delaylp1	decfsz		ICTR,1			; 6..
		goto		delaylp1		; 25

							; fetch data, and place it
		rlf		DATALO,1		; 26
		rlf		DATAHI,1		; 27
		bsf		STATUS,RP0		; 28
		bsf		TRISB,BIT_OSC2		; 29
		btfss		STATUS,C		; 30
		bcf		TRISB,BIT_OSC2		; 31 *
		bcf		STATUS,RP0		; 32
							; ensure data is filled with 1's
		bsf		DATALO,0		; 33
							; wait until 38 to sample data
		nop					; 34
		rlf		FROMSXLO,1		; 35
		rlf		FROMSXHI,1		; 36
		bcf		FROMSXLO,0		; 37
		btfsc		PORTB,BIT_OSC2		; 38 *
		bsf		FROMSXLO,0		; 39
							; wait until around 49 to remove data
		incf		BITCT,1			; 40
		nop					; 41
		nop					; 42
		nop					; 43
		nop					; 44
		nop					; 45
		nop					; 46
		bsf		STATUS,RP0		; 47
		nop					; 48
		bsf		TRISB,BIT_OSC2		; 49 *
		bcf		STATUS,RP0		; 50
;
; the next edge will either arrive at around 78 or, if the
; next edge is a sync, at around 156. we split the difference,
; and decide at 117. enable the interrupts, and see if we
; fall through.
;
		movlw		b'10010000'		; 51
		movwf		INTCON			; 52
		movlw		d'21'			; 53
		movwf		ICTR			; 54
delaylp2	decfsz		ICTR,1			; 55-116
		goto		delaylp2
;
; ok... we fell through - we're in a sync pulse
; we can expect the next edge to arrive at around 156, so we
; have around 39 clocks to do the end-of-word processing
; see if we saw the correct number of bits...
;
		movlw		d'16'
		subwf		BITCT,1			; clear BITCT at same time!
		btfsc		STATUS,Z
		goto		bitct_ok
							; invalid bit count -
							; did we expect it to be correct?
		clrf		BITCT
		btfsc		VALID,0
		goto		bitct_error		; doh!
		goto		wait			; no worries - wait for next pulse

; we received the correct number of bits in the frame
; now, were we actually sending?
;
bitct_ok	btfss		VALID,0
		goto		not_sending
		decf		REPTCT,1		; do we have more frames to transmit?
		btfsc		STATUS,Z
		goto		cmd_ok

not_sending:	bsf		VALID,0			; we expect subsequent frames to be OK
;
; copy data - note that we only start sending once we've seen
; completely valid frame go by...
;
		movf		TDATALO,0
		movwf		DATALO
		movf		TDATAHI,0
		movwf		DATAHI
;
; await next interrupt
; we'll wait for at least a whole frame before declaring a timeout
;
wait		clrf		ICTR
delaylp5	nop
		nop
		nop
		decfsz		ICTR,1
		goto		delaylp5
		goto		timeout			; ok... if we got here,
							; the SX chip just isn't sending

;**************************************************************************************
;
main		call		init_reset		; Reset H/W and S/W
		btfsc		STATREG,0		; Is this a POR
		goto		mainlp

		bsf		STATREG,0		; Indicate POR completion
		bcf		PORTB,1			; Flash VDD LED to indicate COM speed
		call		delay2			; 38.4KBPS = one flash
		bsf		PORTB,1			; 115KBPS  = two flashes
		call		delay2
		btfss		PORTB,7			; PORTB,7 sets the serial speed
		goto		mainlp			; 38.4KBPS mode
		bcf		PORTB,1			; 115KBPS mode
		call		delay2
		bsf		PORTB,1
		call		delay2
		bsf		STATREG,1		; Indicate 115KBPS mode

; the main loop involves receiving a 32-bit command from the PC
; while sending a 16-bit response back. If an SX command is issued,
; the appropriate registers are loaded, and the interrupt routine
; is allowed to take over. It will return via either timeout,
; bitct_error, or cmd_ok.
;
mainlp		movlw		b'00001111'		; build data to transmit
		andwf		FROMSXHI,1		; EEEE:HHHH:LLLL:LLLL
		rlf		ERRORS,1
		rlf		ERRORS,1
		rlf		ERRORS,1
		rlf		ERRORS,0
		andlw		b'11110000'
		iorwf		FROMSXHI,1

		call		xchg_pc_ser
		clrf		ERRORS			; clear errors and response
		clrf		FROMSXLO		; for next time
		clrf		FROMSXHI

		movf		PCCMD,0			; decode command
		sublw		PCCMD_NOP		; NOP command?
		btfsc		STATUS,Z
		goto		mainlp

		movf		PCCMD,0
		sublw		PCCMD_OSC1		; OSC1 voltage level command?
		btfsc		STATUS,Z
		goto		cmd_osc1

		movf		PCCMD,0
		sublw		PCCMD_OSC2		; OSC2 low/tristate command?
		btfsc		STATUS,Z
		goto		cmd_osc2

		movf		PCCMD,0
		sublw		PCCMD_ECHO		; echo command?
		btfsc		STATUS,Z
		goto		cmd_echo

		movf		PCCMD,0
		sublw		PCCMD_SX		; SX command?
		btfsc		STATUS,Z
		goto		cmd_sx

		movf		PCCMD,0
		sublw		PCCMD_PICINIT		; Init PIC for programming?
		btfsc		STATUS,Z
		goto		cmd_picinit

		movf		PCCMD,0
		sublw		PCCMD_PICCMD		; Do a PIC command?
		btfsc		STATUS,Z
		goto		cmd_piccmd

		movf		PCCMD,0
		sublw		PCCMD_PICCMDD		; Do a PIC command with data?
		btfsc		STATUS,Z
		goto		cmd_piccmd

		movf		PCCMD,0
		sublw		PCCMD_PICCLOSE		; Finish programming PIC?
		btfsc		STATUS,Z
		goto		cmd_picclose

		movf		PCCMD,0
		sublw		PCCMD_RESET		; reset programmer?
		btfsc		STATUS,Z
		goto		main

		movf		PCCMD,0
		sublw		PCCMD_PICPROG		; begin/end programming
		btfsc		STATUS,Z		; (for 16C64's ?)
		goto		cmd_picprog

		movf		PCCMD,0
		sublw		PCCMD_VDD		; VDD on/off command?
		btfsc		STATUS,Z
		goto		cmd_vdd

		goto		cmd_invalid

;**************************************************************************************
;
cmd_osc1	movf		PORTA,0
		andlw		b'11111100'
		btfsc		TDATALO,0
		iorlw		b'00000001'
		btfsc		TDATALO,1
		iorlw		b'00000010'
		movwf		PORTA
		goto		mainlp

;**************************************************************************************
;
cmd_osc2	bcf		PORTB,0
		bsf		STATUS,RP0
		btfsc		TDATALO,0
		goto		cmd_osc2_set
		bcf		PORTB,0
		goto		cmd_osc2_done
cmd_osc2_set	bsf		PORTB,0
cmd_osc2_done	bcf		STATUS,RP0
		bcf		PORTB,0
		goto		mainlp

;**************************************************************************************
;
cmd_echo	movf		TDATALO,0
		movwf		FROMSXLO
		movf		TDATAHI,0
		movwf		FROMSXHI
		goto		mainlp

;**************************************************************************************
;
cmd_invalid	movlw		ERROR_CMDINVLD
		movwf		ERRORS
		goto		mainlp

;**************************************************************************************
;
cmd_sx		clrf		VALID			; clear flags
		clrf		BITCT
		bcf		PORTB,0			; ensure B0 is LOW when not tristate
		movlw		h'FF'			; set data to a safe value - 
		movwf		DATAHI			; all 1's initially
		movwf		DATALO
		movlw		b'10010000'		; allow the interrupt processing
		movwf		INTCON			; to take over
		goto		wait

;**************************************************************************************
;
bitct_error	clrf		INTCON			; disable further interrupts
		movlw		ERROR_BITCT		; set error code
		movwf		ERRORS
		goto		mainlp			; continue

;**************************************************************************************
;
timeout		clrf		INTCON			; disable further interrupts
		movlw		ERROR_TIMEOUT		; set error code
		movwf		ERRORS
		goto		mainlp			; continue

;**************************************************************************************
;
cmd_ok		clrf		INTCON			; disable further interrupts
		clrf		ERRORS			; set error code
		goto		mainlp			; continue

;**************************************************************************************
;
; Enable serial mode and send some dummy data
;
sersetup	movlw		d'8'			; send !!!!!!!! to tell the PC to be
		movwf		ICTR			; quiet
shutup_lp	movlw		'!'
		movwf		SER_OUT
		call		serial_send
		clrf		TMP
pause		decfsz		TMP,1
		goto		pause
		decfsz		ICTR,1
		goto		shutup_lp
		return

;**************************************************************************************
;
; B0 and A4 should both be OUTPUT/LOW at this stage...
; we're going to clock 6 bytes of command data from REPTCT, then
; clock 16 bytes of data from TDATAHI/LO - in the case of read commands,
; A4 will be tristate anyway (handy pullup).
;
cmd_piccmd	movlw		d'6'
		movwf		ICTR

pic_cmdlp1	bsf		PORTB,0			; raise clock
		bsf		STATUS,RP0		; access second page
		bsf		TRISA,4			; tristate by default
		rrf		REPTCT,1		; get next bit in carry
		btfss		STATUS,C		; is it clear?
		bcf		TRISA,4			; if so, clear data.
		bcf		STATUS,RP0		; access first page
		bcf		PORTB,0			; lower clock, PIC latches data
		decfsz		ICTR,1			; dec ctr
		goto		pic_cmdlp1		; loop if not zero

		movf		PCCMD,0			; check PC command - data required?
		sublw		PCCMD_PICCMDD
		btfss		STATUS,Z
		goto		done_shortcmd

		bsf		STATUS,RP0		; access second page
		bsf		TRISA,4			; set data tristate... by default
		bcf		STATUS,RP0		; access first page

		movlw		d'16'
		movwf		ICTR

pic_cmdlp2	bsf		PORTB,0			; raise clock
		bsf		STATUS,RP0		; access second page
		bsf		TRISA,4			; tristate by default
		rrf		TDATAHI,1		; rotate data...
		rrf		TDATALO,1		; get next bit in carry
		btfss		STATUS,C		; is it clear?
		bcf		TRISA,4			; if so, clear bit
		bcf		STATUS,RP0		; access first page
		bcf		PORTB,0			; lower clock, PIC latches data
		rrf		FROMSXHI,1		; read data ourselves
		rrf		FROMSXLO,1
		bcf		FROMSXHI,7
		btfsc		PORTA,4
		bsf		FROMSXHI,7

		decfsz		ICTR,1			; dec ctr
		goto		pic_cmdlp2		; loop if not zero

		nop					; ensure PIC has read data
		nop					; (if applicable)
		nop
		nop

		bsf		STATUS,RP0		; access second page
		bcf		TRISA,4			; set A4 to output
		bcf		STATUS,RP0		; access first page
;
; take a copy of FROMSXHI into ERRORS, so it won't get overwritten
; when building the PIC's response to the PC
;
		rrf		FROMSXHI,0
		movwf		ERRORS
		rrf		ERRORS,1
		rrf		ERRORS,1
		rrf		ERRORS,1
		goto		mainlp			; return to main loop

done_shortcmd	bsf		STATUS,RP0		; access second page
		bcf		TRISA,4			; ensure data is output!
		bcf		STATUS,RP0		; access first page
		goto		mainlp


;**************************************************************************************
;
; B0 and A4 should both be OUTPUT/LOW at this stage...
; we're going to clock 6 bits of command data from REPTCT,
; then wait for around 250 clocks (100us).
; then we're going to clock 6 bits of command data from TDATAHI
;
cmd_picprog	movlw		d'6'
		movwf		ICTR
pic_proglp1
		bsf		PORTB,0			; raise clock
		bsf		STATUS,RP0		; access second page
		bsf		TRISA,4			; tristate by default
		rrf		REPTCT,1		; get next bit in carry
		btfss		STATUS,C		; is it clear?
		bcf		TRISA,4			; if so, clear data.
		bcf		STATUS,RP0		; access first page
		bcf		PORTB,0			; lower clock, PIC latches data
		decfsz		ICTR,1			; dec ctr
		goto		pic_proglp1		; loop if not zero

		bsf		STATUS,RP0		; access second page
		bsf		TRISA,4			; set data tristate... by default
		bcf		STATUS,RP0		; access first page

		movlw		d'82'			; delay for 250 clocks
		call		delay
		movlw		d'6'
		movwf		ICTR

pic_proglp2	bsf		PORTB,0			; raise clock
		bsf		STATUS,RP0		; access second page
		bsf		TRISA,4			; tristate by default
		rrf		TDATAHI,1		; rotate data...
		btfss		STATUS,C		; is it clear?
		bcf		TRISA,4			; if so, clear bit
		bcf		STATUS,RP0		; access first page
		bcf		PORTB,0			; lower clock, PIC latches data
		decfsz		ICTR,1			; dec ctr
		goto		pic_proglp2		; loop if not zero

		bsf		STATUS,RP0		; access second page
		bcf		TRISA,4			; set A4 to output
		bcf		STATUS,RP0		; access first page
		goto		mainlp			; return to main loop

;**************************************************************************************
;
; lower A4 and B0 - ensure both are outputs
;
cmd_picinit	bsf		STATUS,RP0		; access second page
		movlw		b'11100000'		; all A outputs
		movwf		TRISA
		movlw		b'11111000'		; B0-2 outputs
		movwf		TRISB
		bcf		STATUS,RP0		; access first page
		bcf		PORTB,0			; lower bits
		bcf		PORTA,4
		goto		mainlp			; done

;**************************************************************************************
;
; return A4 and B0 to tristate
;
cmd_picclose	bsf		STATUS,RP0		; access second page
		movlw		b'11110000'		; A4 now input
		movwf		TRISA
		movlw		b'11111001'		; B0 now input
		movwf		TRISB
		bcf		STATUS,RP0		; access first page
		bcf		PORTB,0			; ensure bits are low
		bcf		PORTA,4
		goto		mainlp			; done

;**************************************************************************************
;
;Turn VDD on/off. New to Ver 2.0
;
cmd_vdd		btfss		TDATALO,0		; TDATALO,0 sets VDD on/off
		bsf		PORTB,1			; VDD off
		btfsc		TDATALO,0
		bcf		PORTB,1			; VDD on

		movlw		.33			; 10mS power on/off delay
		movwf		TMP2
		clrf		TMP
cmd_vddlp	decfsz		TMP
		goto		cmd_vddlp
		decfsz		TMP2
		goto		cmd_vddlp
		goto		mainlp			; done

;**************************************************************************************
;
;Reset after power-up or after cmd_reset
;
init_reset	movlw		0x02			; Initialise IO ports
		movwf		PORTB			; OSC2=0V, VDD=OFF
		bsf		STATUS,RP0		; select bank 1
		movlw		b'11111001'
		movwf		TRISB			; B0=input!
		movlw		b'11110000'		; A=all outputs except A4
		movwf		TRISA
							; Initialise TMR0
		movlw		b'00001000'		; no prescale on tmr0, pullups ENABLED
							; interrupt on B0 falling edge
		movwf		OPTION_REG		; write into option register
		bcf		STATUS,RP0		; select bank 0 again
		clrf		PORTA			; OSC1_5V=0V, OSC1_12V=0V, RS232RX=HI
		clrf		ICTR
init_resetlp1	clrf		BITCT
init_resetlp2	nop
		nop
		nop
		decfsz		BITCT,1
		goto		init_resetlp2
		decfsz		ICTR,1
		goto		init_resetlp1

		clrf		ERRORS			; everything is fine to start with
		clrf		FROMSXLO
		clrf		FROMSXHI
		call		sersetup		; send some dummy serial data
		return

;**************************************************************************************
;
serial_test	call		serial_recv
		btfss		ERRORS,0
		goto		gotone

		movlw		'!';
		movwf		SER_OUT
		call		serial_send
		goto		serial_test

gotone		movf		SER_IN,0
		movwf		SER_OUT
		call		serial_send
		goto		serial_test

;**************************************************************************************
;
; Standard serial transmit at 38400 BPS
;
serial_send	btfsc		STATREG,1		; 38.4 or 115KBPS mode
		goto		fserial_send
		bcf		PORTA,3
		movlw		d'9'			; set up loop ctr
		movwf		BITCT
		goto		ser_clear		; start with START bit

ser_lp		rrf		SER_OUT,1		; 61
		btfsc		STATUS,C		; 62
		goto		ser_set			; 63..64
		nop
ser_clear	bcf		PORTA,2			; 0
		goto		ser_next		; 1..2
ser_set		bsf		PORTA,2			; 0
		goto		ser_next		; 1..2
ser_next	movlw		d'16'			; 3
		call		delay			; 4..55
		nop					; 56
		nop					; 57
		decfsz		BITCT,1			; 58
		goto		ser_lp			; 59..60
		nop					; 61
		nop					; 62
		nop					; 63
		nop					; 64
		bsf		PORTA,2			; 0
		bsf		PORTA,3
		movlw		d'19'			; 1
		call		delay			; 2..62
		return

;**************************************************************************************
;
; Standard serial receive at 38400 BPS
; wait for B4 low, approx 0.13s timeout
;
serial_recv	btfsc		STATREG,1		; 38.4 or 115KBPS mode
		goto		fserial_recv
		clrf		ICTR			; timeout ctr
rtlp1		clrf		TMP			; ctr2
rtlp2		btfss		PORTB,4			; low?
		goto		gotlow			; yeehaw!
		decfsz		TMP,1
		goto		rtlp2
		decfsz		ICTR,1
		goto		rtlp1

recv_err	movlw		d'1'			; timeout - return error
		movwf		ERRORS
		return
;
; got a start bit? assume we caught it at 2... wait to 1/2 bit
;
gotlow		movlw		d'7'			; 3
		call		delay			; 4..28
		nop					; 29
		nop					; 30
		movlw		d'8'			; 31
		movwf		BITCT			; 32
		btfsc		PORTB,4			; 33... ensure still low
		goto		recv_err		; 34

		movlw		d'18'			; 35
		call		delay			; 36..93
		nop					; 94

							; receive bit
ser_rlp		rrf		SER_IN,1		; 63 [95]
		bcf		SER_IN,7		; 64 [96]
		btfsc		PORTB,4			; 0  [97]
		bsf		SER_IN,7		; 1
		nop					; 2
		nop					; 3
		movlw		d'17'			; 4
		call		delay			; 5..59
		decfsz		BITCT,1			; 60
		goto		ser_rlp			; 61..62

		btfss		PORTB,4			; ensure stop bit is set
		goto		recv_err

		movlw		d'7'			; delay 1/2 bit(ish)
		call		delay
		clrf		ERRORS
		return

;**************************************************************************************
;
; Fast serial transmit at 115200 BPS
; 115200 is 21.7 (22) cycles at 10MHz
;
fserial_send	bcf		PORTA,3
		movlw		d'9'			; set up loop ctr
		movwf		BITCT
		goto		fser_clear		; start with START bit

fser_lp		rrf		SER_OUT,1		; 18
		btfsc		STATUS,C		; 19
		goto		fser_set		; 20..21
		nop
fser_clear	bcf		PORTA,2			; 0
		goto		fser_next		; 1..2
fser_set	bsf		PORTA,2			; 0
		goto		fser_next		; 1..2
fser_next	nop					; 3
		nop					; 4
		nop					; 5
		nop					; 6
		nop					; 7
		nop					; 8
		nop					; 9
		nop					; 10
		nop					; 11
		nop					; 12
		nop					; 13
		nop					; 14
		decfsz		BITCT,1			; 15
		goto		fser_lp			; 16..17
		nop					; 18
		nop					; 19
		nop					; 20
		nop					; 21
		bsf		PORTA,2			; 0
		bsf		PORTA,3			; 1
		movlw		d'4'			; 2
		call		delay			; 3..18
		nop					; 19
		return					; 20..21

;**************************************************************************************
;
; Fast serial receive at 115200 BPS
; 115200 is 21.7 (22) cycles at 10MHz
; wait for B4 low, approx 0.13s timeout
;
fserial_recv	clrf		ICTR			; timeout ctr
frtlp1		clrf		TMP			; ctr2
frtlp2		btfss		PORTB,4			; low?
		goto		fgotlow			; yeehaw!
		decfsz		TMP,1
		goto		frtlp2
		decfsz		ICTR,1
		goto		frtlp1

frecv_err	movlw		d'1'			; timeout - return error
		movwf		ERRORS
		return
;
; got a start bit? assume we caught it at 2... wait to 1/2 bit
;
fgotlow		nop					; 3
		nop					; 4
		nop					; 5
		movlw		d'8'			; 6
		movwf		BITCT			; 7
		btfsc		PORTB,4			; 8... ensure still low
		goto		frecv_err		; 9
		movlw		d'3'			; 10
		call		delay			; 11..23
		nop					; 24
		nop					; 25

							; receive bit
fser_rlp	rrf		SER_IN,1		; 20 [26]
		bcf		SER_IN,7		; 21 [27]
		btfsc		PORTB,4			; 0  [28]
		bsf		SER_IN,7		; 1
		nop					; 2
		nop					; 3
		nop					; 4
		nop					; 5
		nop					; 6
		nop					; 7
		nop					; 8
		nop					; 9
		nop					; 10
		nop					; 11
		nop					; 12
		nop					; 13
		nop					; 14
		nop					; 15
		nop					; 16
		decfsz		BITCT,1			; 17
		goto		fser_rlp		; 18..19
		nop
		nop
		btfss		PORTB,4			; ensure stop bit is set
		goto		frecv_err

		movlw		d'3'			; delay 1/2 bit(ish)
		call		delay
		clrf		ERRORS
		return

;**************************************************************************************
;
; delay for 3n+4 (including call/return)
;
delay		movwf		TMP
delaylp		decfsz		TMP,1
		goto		delaylp
		return

; delay for 500mS
;
delay2		clrf		TMP
		clrf		TMP2
		movlw		.7
		movwf		TMP3
delay2lp	decfsz		TMP,1
		goto		delay2lp
		decfsz		TMP2,1
		goto		delay2lp
		decfsz		TMP3,1
		goto		delay2lp
		return

;**************************************************************************************
;
xchg_pc_ser	movlw		'*'
		movwf		SER_OUT
		call		serial_send

ret1		call		serial_recv
		btfsc		ERRORS,0
		goto		ret1
		movf		SER_IN,0
		movwf		PCCMD

		movf		FROMSXHI,0
		movwf		SER_OUT
		call		serial_send

ret2		call		serial_recv
		btfsc		ERRORS,0
		goto		ret2
		movf		SER_IN,0
		movwf		REPTCT

		movf		FROMSXLO,0
		movwf		SER_OUT
		call		serial_send

ret3		call		serial_recv
		btfsc		ERRORS,0
		goto		ret3
		movf		SER_IN,0
		movwf		TDATAHI

		movf		FROMSXLO,0
		xorlw		d'255'
		xorwf		FROMSXHI,0
		movwf		SER_OUT
		call		serial_send

ret4		call		serial_recv
		btfsc		ERRORS,0
		goto		ret4
		movf		SER_IN,0
		movwf		TDATALO
		return

		end

